@@ -1,4 +1,4 @@
-#ident "$Id: class1.c,v 4.10 2006/01/04 21:07:12 gert Exp $ Copyright (c) Gert Doering"
+#ident "$Id: class1.c,v 4.21 2014/04/03 09:04:13 gert Exp $ Copyright (c) Gert Doering"
 
 /* class1.c
  *
@@ -8,6 +8,54 @@
  * Uses library functions in class1lib.c, faxlib.c and modem.c
  *
  * $Log: class1.c,v $
+ * Revision 4.21  2014/04/03 09:04:13  gert
+ * rename FRAMESIZE to C1_FRAMESIZE, collision with AIX header file (grrr)
+ *
+ * Revision 4.20  2009/03/19 15:51:17  gert
+ * bugfix: new NSF sending code was introducing extra "AT+FTH=3" commands
+ * (due to confusion about internal requirements of fax1_send_frame())
+ *
+ * Revision 4.19  2009/03/19 15:33:59  gert
+ * send NSF frames on reception (if modem quirk 0x40, for now)
+ * handle different modem behaviour regarding AT+FTS=8;+FTH=3 gracefully
+ *
+ * Revision 4.18  2006/10/27 09:07:12  gert
+ * add file name of page currently sent to log message
+ *
+ * Revision 4.17  2006/10/25 10:55:01  gert
+ * class 1 receive: log current try #, send DIS only if CSI went without error
+ *
+ * Revision 4.16  2006/09/29 19:30:26  gert
+ * properly calculate scan line time -> padding bytes
+ * implement FTT -> retraining, with baud rate reduction
+ *
+ * Revision 4.15  2006/09/26 15:36:07  gert
+ * - handle modems that don't get AT+FTS=8;+FTM=nn right
+ *   (re-do AT+FTM if the first command yields "OK")
+ * - switch on and off Xon/Xoff flow control before/after sending page data
+ *
+ * Revision 4.14  2006/09/25 22:26:54  gert
+ * fax1_send_page(): move all the G3 file handling to g3file.c functions
+ *  (-> cleanup code, build common infrastructure for class 1 + class 2)
+ *
+ * Revision 4.13  2006/03/22 14:13:12  gert
+ * when sending, hand over received NSF to NSF decoder (faxlib.c/hyla_nsf.c)
+ *
+ * Revision 4.12  2006/03/07 21:31:50  gert
+ * class 1 sending implementation:
+ *   - handle end-of-page (return to phase B or phase C, or send DCN/hangup)
+ *   - move sending of TSI and DCS inside fax1_send_page(), to be able to
+ *     handle RTN/RTP transparently - somewhat sloppy "phase B" definition,
+ *     but much cleaner this way
+ *   - fix reading of G3 files - skip digifax header, also send last chunk
+ *   - fix bit swapping (fax_send_swaptable)
+ * -> class 1 sending now works, if the receiver doesn't need EOL padding
+ *
+ * Revision 4.11  2006/03/07 14:16:56  gert
+ * fax1_dial_and_phase_AB(): add torture test code, refusing incoming CSI/DIS
+ * 2 times before going on (mainly for testing receiver robustness)
+ * fax1_send_page(): add log message to point out unimplemented stuff
+ *
  * Revision 4.10  2006/01/04 21:07:12  gert
  * remove "speed" argument from fax1_send_dcs() (use fax1_max global)
  *
@@ -70,14 +118,22 @@
 #include "fax_lib.h"
 #include "tio.h"
 #include "class1.h"
+#include "policy.h"
 
 enum T30_phases { Phase_A, Phase_B, Phase_C, Phase_D, Phase_E } fax1_phase;
 
+/* maping of "st" codes to actual time (for normal res.) */
+static int fax_scan_times[8] = { 0, 5, 10, 10, 20, 20, 40, 40 };
+
 int fax1_dial_and_phase_AB _P2( (dial_cmd,fd),  char * dial_cmd, int fd )
 {
 char * p;			/* modem response */
-uch framebuf[FRAMESIZE];
+uch framebuf[C1_FRAMESIZE];
 int first;
+int len;
+#ifdef TORTURE_TEST
+int t_tries=0;
+#endif
 
     /* send dial command */
     if ( fax_send( dial_cmd, fd ) == ERROR )
@@ -122,9 +178,11 @@ int first;
      * read all incoming frames until FINAL bit is set
      */
     first=TRUE;
+again:
     do
     {
-	if ( fax1_receive_frame( fd, first? 0:3, 30, framebuf ) == ERROR )
+	if ( (len = fax1_receive_frame( fd, first? 0:3, 30, framebuf ) )
+	       == ERROR )
 	{
 	    /*!!!! try 3 times! (flow diagram from T.30 / T30_T1 timeout) */
 	    fax_hangup = TRUE; fax_hangup_code = 11; return ERROR;
@@ -132,8 +190,9 @@ int first;
 	switch ( framebuf[1] )		/* FCF */
 	{
 	    case T30_CSI: fax1_copy_id( framebuf ); break;
-	    case T30_NSF: break;
+	    case T30_NSF: fax1_incoming_nsf( framebuf+2, len-2 ); break;
 	    case T30_DIS: fax1_parse_dis( framebuf ); break;
+	    case T30_DCN: fax1_send_dcn( fd, 20 ); return ERROR;
 	    default:
 	        lprintf( L_WARN, "unexpected frame type 0x%02x", framebuf[1] );
 	}
@@ -141,14 +200,11 @@ int first;
     }
     while( ( framebuf[0] & T30_FINAL ) == 0 );
 
-    /* send local id frame (TSI) */
-    fax1_send_idframe( fd, T30_TSI|0x01, T30_CAR_V21 );
+#ifdef TORTURE_TEST
+    while( ++t_tries < 3 )
+	{ mdm_command( "AT+FRS=200", fd ); goto again; }
+#endif
 
-    /* send DCS */
-    if ( fax1_send_dcs( fd ) == ERROR )
-    {
-        fax_hangup = TRUE; fax_hangup_code = 10; return ERROR;
-    }
 
     fax1_phase = Phase_B;			/* Phase A done */
 
@@ -159,7 +215,7 @@ int first;
 /* fax1_send_page
  *
  * send a page of G3 data
- * - if phase is "B", include sending of TCF and possibly 
+ * - if phase is "B", include sending of DCS, TCF and possibly 
  *   baud rate stepdown and repeated transmission of DCS.
  * - if phase is "C", directly send page data
  */
@@ -168,22 +224,45 @@ int fax1_send_page _P5( (g3_file, bytes_sent, tio, ppm, fd),
 		        char * g3_file, int * bytes_sent, TIO * tio,
 		        Post_page_messages ppm, int fd )
 {
-uch framebuf[FRAMESIZE];
+uch framebuf[C1_FRAMESIZE];
 char * line;
 char cmd[40];
 char dleetx[] = { DLE, ETX };
 char rtc[] = { 0x00, 0x08, 0x80, 0x00, 0x08, 0x80, 0x00, 0x08 };
-int g3fd, r, w, rx;
-#define CHUNK 512
-char buf[CHUNK], wbuf[CHUNK];
+int pad_bytes, s_time;
 
-    /* if we're in T.30 phase B, send training frame (TCF) now...
+    /* if we're in T.30 phase B, send DCS + training frame (TCF) now...
      * don't forget delay (75ms +/- 20ms)!
+     *
+     * NOTE: this is not strictly "phase B" - in T.30, phase B begins 
+     * after sending the DCS, and before sending TCF.  But since RTN/RTP
+     * return to sending DCS (bullet "D" in T.30 chart 5-2a), grouping it 
+     * this way is much more logical to implement
      */
+
+    /* calculate scan line time
+     * in fine mode, some of the values get divided by 2
+     */
+    s_time = fax_scan_times[ remote_cap.st & 0x7 ];
+    if ( remote_cap.vr == 1 && 
+	(remote_cap.st & 0x1) == 0 ) s_time /= 2;
+
     if ( fax1_phase == Phase_B )
     {
         char train[150];
 	int i, num;
+	int tries=0;
+
+retrain:			/* "(D)" in T.30/Figure 5-2a */
+
+	/* send local id frame (TSI) */
+	fax1_send_idframe( fd, T30_TSI|0x01, T30_CAR_V21 );
+
+	/* send DCS */
+	if ( fax1_send_dcs( fd, s_time ) == ERROR )
+	{
+	    fax_hangup = TRUE; fax_hangup_code = 10; return ERROR;
+	}
 
 	sprintf( cmd, "AT+FTS=8;+FTM=%d", dcs_btp->c_long );
 	fax_send( cmd, fd );
@@ -192,6 +271,18 @@ char buf[CHUNK], wbuf[CHUNK];
 	if ( line != NULL && strcmp( line, cmd ) == 0 )
 		line = mdm_get_line( fd );
 
+	if ( strcmp( line, "OK" ) == 0 )
+	{
+	    lprintf( L_MESG, "fax1_send_page: unexpected OK, re-do AT+FTM" );
+
+	    sprintf( cmd, "AT+FTM=%d", dcs_btp->c_long );
+	    fax_send( cmd, fd );
+
+	    line = mdm_get_line( fd );
+	    if ( line != NULL && strcmp( line, cmd ) == 0 )
+		line = mdm_get_line( fd );
+	}
+
 	if ( line == NULL || strcmp( line, "CONNECT" ) != 0 )
 	{
 	    lprintf( L_ERROR, "fax1_send_page: unexpected response 1: '%s'", line );
@@ -216,16 +307,20 @@ char buf[CHUNK], wbuf[CHUNK];
 	}
 
 	/* receive frame - FTT or CFR */
-	/*!!! return code! */
+	/*!!! return code! TODO: retry logic - T.30 fig. 5-2a/p.16 */
 	fax1_receive_frame( fd, 3, 30, framebuf );
 
 	if ( ( framebuf[0] & T30_FINAL ) == 0 ||
 	     framebuf[1] != T30_CFR )
 	{
-	    lprintf( L_ERROR, "fax1_receive_frame: failed to train" );
-	    /*!!! try 3 times! */
-	    fax_hangup = TRUE; fax_hangup_code = 27;
-	    return ERROR;
+	    tries++;
+	    lprintf( L_WARN, "fax1_send_frame: failed to train (%d)", tries);
+
+	    if ( tries > 1 ) fax1_reduce_max();
+	    if ( tries < 3 ) goto retrain;
+
+	    /* give up */
+	    fax1_send_dcn( fd, 27 ); return ERROR;
 	}
 
 	/* phase B done, go to phase C */
@@ -239,16 +334,21 @@ char buf[CHUNK], wbuf[CHUNK];
 	return ERROR;
     }
 
-    r=0;w=0;
-    g3fd = open( g3_file, O_RDONLY );
-    if ( g3fd < 0 )
+    /* open G3 file, read first chunk, potentially skipping digifax header
+     */
+    if ( g3_open_read( g3_file ) < 0 )
     {
-        lprintf( L_ERROR, "fax1_send_page: can't open '%s'", g3_file );
-	/*!!! do something smart here... */
+	/*!!! do something smart here...? */
 	fax1_send_dcn( fd, FHUP_ERROR );
 	return ERROR;
     }
 
+
+    /* number of bytes to pad depend on scan time */
+    pad_bytes = ((dcs_btp->speed/8) * s_time + 999) / 1000;
+    lprintf( L_MESG, "scan line time: %d ms -> %d bytes/line at %d bps",
+		s_time, pad_bytes, dcs_btp->speed );
+
     /* Phase C: send page data with high-speed carrier
      */
     sprintf( cmd, "AT+FTM=%d", dcs_btp->c_short );
@@ -264,52 +364,31 @@ char buf[CHUNK], wbuf[CHUNK];
 	fax_hangup = TRUE; fax_hangup_code = 40; return ERROR;
     }
 
-    lprintf( L_NOISE, "send page data" );
+    lprintf( L_NOISE, "send page data (\"%s\")...", g3_file );
+
+    /* turn on xon/xoff flow control now, for page data sending
+     */
+    if ( (FAXSEND_FLOW) & FLOW_SOFT )
+    {
+	tio_set_flow_control( fd, tio, (FAXSEND_FLOW) & (FLOW_HARD|FLOW_XON_OUT));
+	tio_set( fd, tio );
+    }
 
     /* read page data from file, invert byte order, 
      * insert padding bits (if scan line time > 0), 
      * at end-of-file, add RTC
      */
-    /*!!!! padding, one-line-at-a-time, watch out for sizeof(wbuf)*/
-    /*!!!! digifax header!*/
-    rx=0; r=0; w=0;
-    do
+    if ( g3_send_file( g3_rf_chunk, fd, TRUE, TRUE, pad_bytes, 0 /*TODO!*/) < 0 )
     {
-        if ( rx >= r )			/* buffer empty, read more */
-	{
-	    r = read( g3fd, buf, CHUNK );
-	    if ( r < 0 )
-	    {
-	    	lprintf( L_ERROR, "fax1_send_page: error reading '%s'", g3_file );
-		break;
-	    }
-	    if ( r == 0 ) break;
-	    lprintf( L_JUNK, "read %d", r );
-	    rx = 0;
-	}
-	wbuf[w] = buf[rx++];
-	if ( wbuf[w] == DLE ) wbuf[++w] = DLE;
-	w++;
-
-	/*!! zero-counting, bitpadding! */
-	if ( w >= sizeof(wbuf)-2 )
-	{
-	    if ( w != write( fd, wbuf, w ) )
-	    {
-	        lprintf( L_ERROR, "fax1_send_page: can't write %d bytes", w );
-		break;
-	    }
-	    lprintf( L_JUNK, "write %d", w );
-	    w=0;
-	}
+	lprintf( L_ERROR, "error in g3_send_file()" ); 
+	return ERROR;
     }
-    while(r>0);
-    close(g3fd);
 
     /*!!! ERROR HANDLING!! */
     /*!!! PARANOIA: alarm()!! */
     /* end of page: RTC */
     write( fd, rtc, sizeof(rtc) );
+
     /* end of data: DLE ETX */
     write( fd, dleetx, 2 );
 
@@ -320,6 +399,12 @@ char buf[CHUNK], wbuf[CHUNK];
 	fax_hangup = TRUE; fax_hangup_code = 40; return ERROR;
     }
 
+    /* turn off xon/xoff flow control (will interfere with received frames) */
+    if ( (FAXSEND_FLOW) & FLOW_SOFT )
+    {
+	tio_set_flow_control( fd, tio, (FAXSEND_FLOW) & FLOW_HARD );
+	tio_set( fd, tio );
+    }
 
     /* now send end-of-page frame (MPS/EOM/EOP) and get pps */
 
@@ -354,7 +439,14 @@ tryanyway:
 	   lprintf( L_WARN, "fax1_send_page: canthappen(1) - PRI not supported" );
     }
 
-    fax1_send_frame( fd, strcmp(line, "OK")==0? 3:0 , framebuf, 2 );
+    if ( strcmp( line, "OK" ) == 0 )	/* re-send AT+FTH=3 */
+    {
+        fax1_send_frame( fd, T30_CAR_V21, framebuf, 2 );
+    }
+    else				/* it got sent & acked */
+    {
+        fax1_send_frame( fd, T30_CAR_HAVE_V21, framebuf, 2 );
+    }
 
     /* get MPS/RTP/RTN code */
     fax1_receive_frame( fd, T30_CAR_V21, 30, framebuf );
@@ -364,22 +456,35 @@ tryanyway:
     switch( framebuf[1] )
     {
         case T30_MCF:		/* page good */
-		fax_page_tx_status = 1; break;
+		fax_page_tx_status = 1; fax1_phase = Phase_C; break;
 	case T30_RTN:		/* retrain / negative */
 		fax_page_tx_status = 2; fax1_phase = Phase_B; break;
 	case T30_RTP:		/* retrain / positive */
 		fax_page_tx_status = 3; fax1_phase = Phase_B; break;
 	case T30_PIN:		/* procedure interrupt */
-		fax_page_tx_status = 4; break;
+		fax_page_tx_status = 4; fax1_phase = Phase_B; break;
 	case T30_PIP:
-		fax_page_tx_status = 5; break;
+		fax_page_tx_status = 5; fax1_phase = Phase_C; break;
 	default:
 		lprintf( L_ERROR, "fax1_transmit_page: unexpected frame" );
 		fax1_send_dcn(fd, 53); break;
     }
 
-    fax_hangup = TRUE; fax_hangup_code = 50;
-    return ERROR;
+    /* if this was the last page, and the receiver is happy
+     * (MCF or RTP), send DCN, and call it quits
+     */
+    if ( ppm == pp_eop && 
+	 ( fax_page_tx_status == 1 || fax_page_tx_status == 3 || 
+	   fax_page_tx_status == 5 ) )
+    {
+	fax1_send_dcn( fd, 0 ); 
+	fax1_phase = Phase_E;
+    }
+
+    /* otherwise, nothing to do - caller will re-enter fax1_send_page(),
+     * for next page / re-send of same page (we won't know)
+     */
+    return NOERROR;
 }
 
 boolean fax1_receive_have_connect = FALSE;
@@ -393,7 +498,7 @@ int fax1_highlevel_receive _P6( (fd, pagenum, dirlist, uid, gid, mode ),
 			       int uid, int gid, int mode)
 {
 int rc;
-uch frame[FRAMESIZE];
+uch frame[C1_FRAMESIZE];
 char * p;
 Post_page_messages ppm = pp_mps;
 boolean first = TRUE;
@@ -432,13 +537,21 @@ boolean have_dcs;
     tries=0; have_dcs=FALSE;
 
 receive_phase_b:
-    lprintf( L_MESG, "fax1 T.30 receive phase B" );
+    lprintf( L_MESG, "fax1 T.30 receive phase B (try %d)", tries+1 );
+
+    /* send NSF frame (if requested) */
+    if ( modem_quirks & MQ_SHOW_NSF )
+    {
+	rc = fax1_send_nsf( fd, first?T30_CAR_HAVE_V21: T30_CAR_V21 );
+    }
 
     /* send local ID frame (CSI) - non-final */
-    fax1_send_idframe( fd, T30_CSI, first?T30_CAR_SAME: T30_CAR_V21 );
+    rc = fax1_send_idframe( fd, T30_CSI, first?T30_CAR_HAVE_V21: T30_CAR_V21 );
     first = FALSE;
 
-    rc = fax1_send_dis( fd );
+    /* send DIS (but only if we haven't seen an error sending CSI) */
+    if ( rc == NOERROR )
+        rc = fax1_send_dis( fd );
 
     /* now see what the other side has to say... */
 wait_for_dcs:
