@@ -1,8 +1,8 @@
-#! /usr/local/bin/wish3.6 -f
+#! /usr/local/bin/wish
 #
 # faxview.tcl -- a simple dialog for viewing FAX messages.
 #
-# Copyright (C) 1994, 1995 Ralph Schleicher <rs@purple.in-ulm.de>
+# Copyright (C) 1994--1996 Ralph Schleicher <rs@purple.in-ulm.de>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -79,89 +79,58 @@ set color_ground	#dfdfbf		;# gray-shaded ivory
 set color_input		#ffffdf		;# light yellow
 set color_selection	#ffdfbf		;# pale orange
 set color_button	#bf5f5f		;# indian red
+set color_highlight	$color_disabled
 
 . configure \
 	-background $color_background
 
-option add Tk*background $color_background
-option add Tk*foreground $color_foreground
-option add Tk*activeBackground $color_active
-option add Tk*activeForeground $color_foreground
-option add Tk*selectBackground $color_selection
-option add Tk*selectForeground $color_foreground
-option add Tk*insertBackground $color_foreground
-option add Tk*disabledForeground $color_disabled
-option add Tk*selector $color_button
+option add *background $color_background
+option add *foreground $color_foreground
+option add *activeBackground $color_active
+option add *activeForeground $color_foreground
+option add *selectBackground $color_selection
+option add *selectForeground $color_foreground
+option add *insertBackground $color_foreground
+option add *disabledForeground $color_disabled
+option add *highlightColor $color_highlight
+option add *troughColor $color_ground
+option add *selector $color_button
 
-option add Tk*Label.anchor w
+option add *highlightThickness 0
+option add *insertWidth 1
 
-option add Tk*Message.anchor w
+option add *Label.anchor w
 
-option add Tk*Entry.relief sunken
-option add Tk*Entry.borderWidth 2
-option add Tk*Entry.background $color_input
+option add *Message.anchor w
 
-option add Tk*Listbox.relief sunken
-option add Tk*Listbox.borderWidth 2
-option add Tk*Listbox.background $color_input
+option add *Entry.relief sunken
+option add *Entry.borderWidth 2
+option add *Entry.background $color_input
 
-option add Tk*Scrollbar.relief sunken
-option add Tk*Scrollbar.borderWidth 2
-option add Tk*Scrollbar.width 15
-option add Tk*Scrollbar.activeForeground $color_background
-option add Tk*Scrollbar.foreground $color_background
-option add Tk*Scrollbar.background $color_ground
+option add *Listbox.relief sunken
+option add *Listbox.borderWidth 2
+option add *Listbox.background $color_input
 
-option add Tk*Scale.activeForeground $color_background
-option add Tk*Scale.foreground $color_background
-option add Tk*Scale.background $color_ground
+option add *Scrollbar.relief sunken
+option add *Scrollbar.borderWidth 2
+option add *Scrollbar.width 15
 
-option add Tk*Menu.activeBorderWidth 2
-option add Tk*Menu.activeBackground $color_background
+option add *Menu.activeBorderWidth 2
+option add *Menu.activeBackground $color_background
 
-option add Tk*Menubutton.activeBackground $color_background
-option add Tk*Menubutton.borderWidth 2
+option add *Menubutton.activeBackground $color_background
+option add *Menubutton.borderWidth 2
 
-option add Tk*Button.width 7
-option add Tk*Button.padX 2
-option add Tk*Button.padY 2
+option add *Button.width 7
+option add *Button.padX 2
+option add *Button.padY 2
+option add *Button.highlightThickness 1
 
-option add Tk*Radiobutton.relief flat
-option add Tk*Radiobutton.anchor w
+option add *Radiobutton.relief flat
+option add *Radiobutton.anchor w
 
-option add Tk*Checkbutton.relief flat
-option add Tk*Checkbutton.anchor w
-
-# Change the active behaviour of buttons a little bit.
-#
-proc button_down win {
-
-    global tk_priv
-
-    set tk_priv(buttonWindow) $win
-}
-
-proc button_up win {
-
-    global tk_priv
-
-    if {$win == $tk_priv(buttonWindow)} {
-
-	if {($win == $tk_priv(window)) \
-		&& ([lindex [$win configure -state] 4] != "disabled")} {
-
-	    uplevel #0 [list $win invoke]
-	}
-
-	set tk_priv(buttonWindow) ""
-    }
-}
-
-bind Radiobutton <ButtonPress-1> {button_down %W}
-bind Radiobutton <ButtonRelease-1> {button_up %W}
-
-bind Checkbutton <ButtonPress-1> {button_down %W}
-bind Checkbutton <ButtonRelease-1> {button_up %W}
+option add *Checkbutton.relief flat
+option add *Checkbutton.anchor w
 
 
 # --------------------------------------------------------------
@@ -354,7 +323,7 @@ proc log_enter {args} {
 
 proc log_end {} {
 
-    if {[.log.text get {end - 1 char}] != "\n"} {
+    if {[.log.text get {end lineend}] != "\n"} {
 
 	.log.text insert end "\n"
     }
@@ -654,7 +623,7 @@ proc do_update {} {
 	.dialog.dirs.list insert end ".."
     }
 
-    foreach f $match {
+    foreach f [lsort $match] {
 
 	if [file isdirectory $f] {
 
@@ -686,7 +655,7 @@ proc set_name {} {
 
     global curr_name
 
-    set curr_name [selection get]
+    regsub -all "\n" [selection get] " " curr_name
 }
 
 
@@ -757,8 +726,8 @@ pack .dialog.name [frame .dialog.gap -height 8] .dialog.glob \
 pack [frame .dialog.listboxes] \
 	-fill both
 
-xlistbox .dialog.files Files: -geometry 32x8
-xlistbox .dialog.dirs Directories: -geometry 32x8
+xlistbox .dialog.files Files: -width 32 -height 8 -selectmode extended
+xlistbox .dialog.dirs Directories: -width 32 -height 8 -selectmode single
 
 pack .dialog.files .dialog.dirs \
 	-in .dialog.listboxes \
@@ -777,9 +746,6 @@ pack [frame .command -borderwidth 4] \
 pack [xentry .command.view {Viewing command:} view_command] \
 	-fill x
 
-pack [xentry .command.print {Printing command:} print_command] \
-	-fill x
-
 checkbutton .command.view_single \
 	-text {Single filenames} -variable view_single
 
@@ -787,6 +753,9 @@ place .command.view_single \
 	-in .command.view.entry \
 	-relx 1.0 -y -2 -anchor se
 
+pack [xentry .command.print {Printing command:} print_command] \
+	-fill x
+
 checkbutton .command.print_single \
 	-text {Single filenames} -variable print_single
 
@@ -869,16 +838,12 @@ buttons	View do_view \
 
 bind .dialog.name.entry <Return> {.button.view invoke}
 bind .dialog.name.entry <Escape> {.button.exit invoke}
-bind .dialog.name.entry <Tab> {focus .dialog.glob.entry}
 bind .dialog.glob.entry <Return> {.button.update invoke}
 bind .dialog.glob.entry <Escape> {.button.exit invoke}
-bind .dialog.glob.entry <Tab> {focus .command.view.entry}
 bind .command.view.entry <Return> {.button.view invoke}
 bind .command.view.entry <Escape> {.button.exit invoke}
-bind .command.view.entry <Tab> {focus .command.print.entry}
 bind .command.print.entry <Return> {.button.print invoke}
 bind .command.print.entry <Escape> {.button.exit invoke}
-bind .command.print.entry <Tab> {focus .dialog.name.entry}
 
 bind .dialog.files.list <ButtonRelease-1> \
 	{set_name}
