@@ -1,4 +1,4 @@
-#ident "$Id: mg_m_init.c,v 4.7 1999/03/13 19:21:42 gert Exp $ Copyright (c) Gert Doering"
+#ident "$Id: mg_m_init.c,v 4.12 2006/01/03 10:15:37 gert Exp $ Copyright (c) Gert Doering"
 
 /* mg_m_init.c - part of mgetty+sendfax
  *
@@ -18,6 +18,8 @@
 #endif
 
 #ifdef linux
+# include <sys/types.h>
+typedef u_int32_t __u32;
 # include <linux/serial.h>
 #endif
 
@@ -121,6 +123,21 @@ int mg_init_fax _P5( (fd, mclass, fax_id, fax_only, fax_max_speed),
 	lprintf( L_NOISE, "no class 2/2.0 faxmodem, no faxing available" );
 	return FAIL;
     }
+
+    if ( modem_type == Mt_class1 || modem_type == Mt_class1_0 )
+    {
+	/* set adaptive answering */
+        char aabuf[20];
+	sprintf( aabuf, "AT+FA%s=%d",
+			modem_type == Mt_class1? "E": "A",
+			fax_only? 0:1 );
+	if ( mdm_command( aabuf, fd ) == FAIL )
+	{
+	    lprintf( L_MESG, "cannot set answer/reception flags" );
+	}
+
+	fax_set_bor( fd, 1 );		/* can't fail */
+    }
     
     if ( modem_type == Mt_class2_0 )
     {
@@ -246,6 +263,7 @@ int mg_open_device _P2 ( (devname, blocking),
     else		/* blocking open */
     {
       again:
+	lprintf( L_MESG, "mod: blocking-open(%s)", devname );
 	fd = open( devname, O_RDWR | O_NOCTTY );
 	    
 	if ( fd < 0)
