@@ -4,7 +4,7 @@
  * pvftormd converts from the pvf (portable voice format) format to the
  * rmd (raw modem data) format.
  *
- * $Id: pvftormd.c,v 1.18 2004/07/17 15:53:02 gert Exp $
+ * $Id: pvftormd.c,v 1.19 2005/03/13 17:27:48 gert Exp $
  *
  */
 
@@ -55,6 +55,7 @@ static void supported_formats (void)
      fprintf(stderr, " - V253modem      8        8-bit linear unsigned PCM\n");
      fprintf(stderr, " - V253modem      9        8-bit linear signed PCM\n");
      fprintf(stderr, " - V253modem      12       16-bit linear signed PCM Intel Order\n");
+     fprintf(stderr, " - V253modem      13       16-bit linear unsigned PCM Intel Order\n");
      fprintf(stderr, " - ZyXEL_1496     2, 3, 4  2/3/4-bit ZyXEL ADPCM\n");
      fprintf(stderr, " - ZyXEL_2864     2, 3, 4  2/3/4-bit ZyXEL ADPCM\n");
      fprintf(stderr, " - ZyXEL_2864     81       8-bit Rockwell PCM\n");
@@ -898,6 +899,25 @@ int main (int argc, char *argv[])
 
           };
 
+     if ((strcmp(modem_type, "V253modem") == 0) && (compression == 13))
+          {
+          header_out.bits = 16;
+
+          if (write_rmd_header(fd_out, &header_out) != OK)
+               {
+               fclose(fd_out);
+
+               if (fd_out != stdout)
+                    unlink(name_out);
+
+               exit(ERROR);
+               };
+          /* Unsigned PCM 16-bit Intel bit ordering */
+          if (pvftolin(fd_in, fd_out, &header_in, 0, 16, 1) == OK)
+               exit(OK);
+
+          };
+
      if ((strcmp(modem_type, "ZyXEL 1496") == 0) &&
       ((compression == 2) || (compression == 3) || (compression == 4)))
           {
